% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BASiCS_diagPlot.R
\name{BASiCS_diagPlot}
\alias{BASiCS_diagPlot}
\title{Create diagnostic plots of MCMC parameters}
\usage{
BASiCS_diagPlot(object, Param = "mu", x = NULL, y = NULL,
  LogX = isTRUE(x \%in\% c("mu", "delta")), LogY = isTRUE(y \%in\%
  c("mu", "delta")), na.rm = TRUE)
}
\arguments{
\item{object}{an object of class \code{\linkS4class{BASiCS_Summary}}}

\item{Param}{Optional name of a chain parameter to restrict the histogram;
if not supplied, all parameters will be assessed.
Possible values: \code{'mu'}, \code{'delta'}, \code{'phi'},
\code{'s'}, \code{'nu'}, \code{'theta'}, \code{'beta'},
\code{'sigma2'} and \code{'epsilon'}. Default \code{Param = 'mu'}}

\item{x, y}{Optional MCMC parameter values to be plotted on the x or y axis, 
respectively. If neither is supplied, Param will be plotted on the x axis
and \code{coda::effectiveSize(Param)} will be plotted on the y axis as
a density plot.}

\item{LogX, LogY}{A boolean value indicating whether to use a log10
transformation for the x or y axis, respectively.}

\item{na.rm}{Logical value indicating whether NA values should be removed
before calculating effective sample size.}
}
\value{
A ggplot object.
}
\description{
Plot parameter values and effective sample size.
See \link[coda]{effectiveSize}
for more details on this diagnostic measure.
}
\examples{

# Built-in example chain
data(ChainSC)

# Point estimates versus effective sample size
BASiCS_diagPlot(ChainSC, Param = "mu")
# Effective sample size as colour, mu as x, delta as y.
BASiCS_diagPlot(ChainSC, x = "mu", y = "delta")

}
\seealso{
\code{\linkS4class{BASiCS_Chain}}
}
\author{
Alan O'Callaghan \email{a.b.ocallaghan@sms.ed.ac.uk}
}
